/*
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        28.05.2004
 Description:    Dieses Programm dient als Beispiel zur Ansteuerung einer MMC/SD-Memory-Card.
				 Zum Zugriff auf eine MMC/SD-Karte, mu man nur die Datei mmc.c
				 in sein eigenes Programm einfgen.
*/

#include <stdio.h>
#include <avr/io.h>


//Anpassen der Register wenn ein ATMega128 benutzt wird
#if defined (__AVR_ATmega128__)
#	define USR UCSR0A
#	define UCR UCSR0B
#	define UDR UDR0
#	define UBRR UBRR0L
#	define EICR EICRB
#endif

//Anpassen der Register wenn ein ATMega8 benutzt wird
#if defined (__AVR_ATmega8__)
#	define USR UCSRA
#	define UCR UCSRB
#	define UBRR UBRRL
#endif

//Der Quarz auf dem Experimentierboard hat eine Frequenz von 14.318MHz
#define sysclk	4000000	//Quarz Frequenz in Hz

//Die Baud_Rate der Seriellen Schnittstelle ist 9600 Baud
#define baud_rate 9600		//Baud Rate fr die Serielle Schnittstelle	

#include <mmc.c>


//Installation der Seriellen Schnittstelle
void IOInit (void)
{
	//Enable TXEN im Register UCR TX-Data Enable
	outp ((1 << TXEN),UCR);
	//Teiler wird gesetzt 
	outp ((sysclk / (baud_rate * 16L) - 1) , UBRR);
}


//Routine fr printf
int uart_putchar (char c)
{
	if (c == '\n')
		uart_putchar('\r');
	//Warten solange bis Zeichen gesendet wurde
	loop_until_bit_is_set(USR, UDRE);
	UDR = c;
	return (0);
}

//Hauptprogramm
void main (void)
{
	//Initzialisierung der seriellen Schnittstelle
	IOInit();
	
	//ffnet einen kanal fr printf
	fdevopen (uart_putchar, NULL, 0);
	printf ("Startup OK\nInitializing MMC/SD-Card!\n");

	//Initialisierung der MMC/SD-Karte
	char Timeout = 0;
	char a = 1;	
	while (a !=0) /*ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten
					MMC/SD-Karte wird dann neu Initialisiert*/
	{
		a = Init_MMC();	
		Timeout++;
		if (Timeout == 200)
			{
			printf ("Keine MMC/SD-Karte gefunden!!\n");
			while (1) {}
			}
	}
	//Initialisierung der MMC/SD-Karte ENDE!


	char Buffer[512];
	long addr = 0;
	char tmp = 0;


	//Erzeugt ein  Array von 512 Bytes mit dem Inhalt EEhex
	for (int a=0;a<512;a=a+1)
	{
	Buffer[a] = 0xEE;
	}


	//schreibt Array in den ersten Blocks der MMC/SD Karte (addr = 0)
	tmp = Write_Block_MMC (addr,Buffer);
	if (tmp != 0) //ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten 
		{
		printf("Error %x\n",tmp);
		}	


	//auslesen des zweiten Blocks von der MMC/SD Karte (addr = 1)
	addr = 1;
	tmp = Read_Block_MMC (addr,Buffer);
	if (tmp != 0) //ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten 
		{
		printf("Error %x\n",tmp);
		}
	//ausgabe des zweiten Blocks auf die serielle Schnittstelle
	for (int a=0;a<512;a=a+8)
	{
	printf("%x %x %x %x %x %x %x %x ",Buffer[a],Buffer[a+1],Buffer[a+2],Buffer[a+3],Buffer[a+4],Buffer[a+5],Buffer[a+6],Buffer[a+7]);
	a=a+8;
	printf("%x %x %x %x %x %x %x %x\n",Buffer[a],Buffer[a+1],Buffer[a+2],Buffer[a+3],Buffer[a+4],Buffer[a+5],Buffer[a+6],Buffer[a+7]);
	}


	//auslesen des ersten Blocks von der MMC/SD Karte (addr = 1)
	addr = 0;
	tmp = Read_Block_MMC (addr,Buffer);
	if (tmp != 0) //ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten 
		{
		printf("Error %x\n",tmp);
		}	
	//ausgabe des ersten Blocks auf die serielle Schnittstelle
	for (int a=0;a<512;a=a+8)
	{
	printf("%x %x %x %x %x %x %x %x ",Buffer[a],Buffer[a+1],Buffer[a+2],Buffer[a+3],Buffer[a+4],Buffer[a+5],Buffer[a+6],Buffer[a+7]);
	a=a+8;
	printf("%x %x %x %x %x %x %x %x\n",Buffer[a],Buffer[a+1],Buffer[a+2],Buffer[a+3],Buffer[a+4],Buffer[a+5],Buffer[a+6],Buffer[a+7]);
	}	


	//Hauptprogramm luft stndig in einer schleife und macht nichts
	while (1)
		{
		}
}

